import {
  SAVE_SETTINGS, UPDATE_DATE_RANGE, MERGE_BOOKINGS,
  UPDATE_BOOKING, DELETE_BOOKING,
  TOGGLE_MODAL, AUTOCOMPLETE_MODAL,
  MODAL_STATE_SELECT,
  MODAL_START_DATE_SELECT, MODAL_START_TIME_SELECT,
  MODAL_END_DATE_SELECT, MODAL_END_TIME_SELECT,
  MODAL_STATUS,
} from "./actionTypes";

import moment from 'moment';

export const saveSettings = (dataUnitId, dataEventType, dataStates, dataLanguage, dataStartTime, dataEndTime, dataRefreshInterval, calendarDefaultView) => ({
  type: SAVE_SETTINGS,
  payload: {
    dataUnitId,
    dataEventType,
    dataStates,
    dataLanguage,
    dataStartTime,
    dataEndTime,
    dataRefreshInterval,
    calendarDefaultView,
  }
});

export const getBookings = (dataUnitId, dataEventType, startTime, endTime, oldestBooking) => {
  return dispatch => {
    nextpath_client.getBookings(dataUnitId, dataEventType, startTime, endTime, oldestBooking, (bookings) => (
      dispatch(parseBookings(bookings))
    ));
  };
};

export const parseBookings = (bookings) => {
  return dispatch => {
    if (bookings.hasOwnProperty('data')) {
      var newBookings = nextpath_helpers.parseBookings(bookings);
      // Update the bookings list if we received new booking from the server.
      if (newBookings.length > 0) {
        dispatch(mergeBookings(newBookings));
      }
    }
  };
}

export const mergeBookings = (bookings) => ({
  type: MERGE_BOOKINGS,
  payload: {
    bookings,
  },
});

export const editBookingInModal = (booking) => {
  return dispatch => {
    dispatch(loadBookingIntoModal(booking, null));
    dispatch(openModal());
  };
}

export const createBookingInModal = ({ start, end }) => {
  return dispatch => {
    dispatch(loadBookingIntoModal(null, { start, end }));
    dispatch(openModal());
  };
}

export const loadBookingIntoModal = (booking, slot) => {
  return dispatch => {
    if (booking == null) {
      // We create a new booking.
      dispatch(autocompleteModal({
        modalBookingId: null,
        modalTitle: Drupal.t('Create booking'),
        modalStateSelect: '',
        modalStartDate: moment(slot.start).format('YYYY-MM-DD'),
        modalStartTime: moment(slot.start).format('HH:mm'),
        modalEndDate: moment(slot.end).format('YYYY-MM-DD'),
        modalEndTime: moment(slot.end).format('HH:mm'),
      }));
    }
    else if (slot == null) {
      // We edit an existing booking.
      dispatch(autocompleteModal({
        modalBookingId: booking.id,
        modalTitle: Drupal.t('Edit booking'),
        modalStateSelect: booking.stateId,
        modalStartDate: moment(booking.start).format('YYYY-MM-DD'),
        modalStartTime: moment(booking.start).format('HH:mm'),
        modalEndDate: moment(booking.end).format('YYYY-MM-DD'),
        modalEndTime: moment(booking.end).format('HH:mm'),
      }));
    }
  };
}

export const openModal = () => ({
  type: TOGGLE_MODAL,
  payload: {
    modalIsOpen: true,
  },
});

export const closeModal = () => ({
  type: TOGGLE_MODAL,
  payload: {
    modalIsOpen: false,
  },
});

export const autocompleteModal = (modal) => ({
  type: AUTOCOMPLETE_MODAL,
  payload: {
    modalBookingId: modal.modalBookingId,
    modalTitle: modal.modalTitle,
    modalStateSelect: modal.modalStateSelect,
    modalStartDate: modal.modalStartDate,
    modalStartTime: modal.modalStartTime,
    modalEndDate: modal.modalEndDate,
    modalEndTime: modal.modalEndTime,
  },
});

export const handleModalSelectChange = (e) => ({
  type: MODAL_STATE_SELECT,
  payload: {
    modalStateSelect: e.target.value,
  },
});

export const handleStartDateChange = (e) => ({
  type: MODAL_START_DATE_SELECT,
  payload: {
    modalStartDate: e.target.value,
  },
});

export const handleStartTimeChange = (e) => ({
  type: MODAL_START_TIME_SELECT,
  payload: {
    modalStartTime: e.target.value,
  },
});

export const handleEndDateChange = (e) => ({
  type: MODAL_END_DATE_SELECT,
  payload: {
    modalEndDate: e.target.value,
  },
});

export const handleEndTimeChange = (e) => ({
  type: MODAL_END_TIME_SELECT,
  payload: {
    modalEndTime: e.target.value,
  },
});

export const handleBookingSave = (settings, modal, oldestBooking) => {
  return dispatch => {
    var error = nextpath_helpers.validateBooking(modal.modalStartDate, modal.modalStartTime, modal.modalEndDate, modal.modalEndTime, modal.modalStateSelect);
    if (error.length == 0) {
      // If no error, go on with saving.
      var startDate = new Date(modal.modalStartDate + ' ' + modal.modalStartTime);
      var endDate = new Date(modal.modalEndDate + ' ' + modal.modalEndTime);
      // Pass the form values to the helper function, which will prepare the
      // JSON.
      const b = nextpath_helpers.createBooking(settings.dataUnitId, settings.dataEventType, modal.modalStateSelect, nextpath_helpers.formatDate(startDate), nextpath_helpers.formatDate(endDate), modal.modalBookingId);
      // After passing the booking to the server, we need to update the state.
      nextpath_client.createBooking(b, settings.dataEventType, (reply) => (
        dispatch(handleBookingAfterSave(b, reply, settings, modal, oldestBooking))
      ));
      dispatch(setModalStatus('', ['alert']));
      dispatch(closeModal());
    }
    else {
      // Display the error messages.
      dispatch(setModalStatus(error.join(', '), ['alert', 'alert-danger']));
    }
  };
}

export const handleBookingAfterSave = (booking, reply, settings, modal, oldestBooking) => {
  return dispatch => {
    // We made an update of a booking, update the state as well.
    if (booking.data.hasOwnProperty('id')) {
      dispatch(updateBooking(booking, reply));
    }
    // We created a new booking, load the new bookings from server.
    else {
      dispatch(getBookings(settings.dataUnitId, settings.dataEventType, settings.dataStartTime, settings.dataEndTime, oldestBooking));
    }
  };
}

export const setModalStatus = (statusMessage, statusClass) => ({
  type: MODAL_STATUS,
  payload: {
    statusMessage,
    statusClass,
  },
});

export const updateBooking = (booking, reply) => ({
  type: UPDATE_BOOKING,
  payload: {
    booking,
    reply,
  },
});

export const handleBookingDelete = (settings, modal) => {
  return dispatch => {
    // Display a confirmation window for the delete.
    var confirmResult = confirm(Drupal.t('Do you really want to delete the booking?'));
    if (confirmResult == true) {
      nextpath_client.deleteBooking(modal.modalBookingId, settings.dataEventType, (reply) => (
        dispatch(deleteBooking(modal.modalBookingId))
      ));
    }
    dispatch(closeModal());
  };
}

export const deleteBooking = (id) => ({
  type: DELETE_BOOKING,
  payload: {
    id,
  },
});

export const handleCalendarRangeChange = (range, settings, oldestBooking) => {
  return dispatch => {
    // Get the start and end time from the range selected.
    var newStart = null;
    var newEnd = null;
    if (range.hasOwnProperty('start') && range.hasOwnProperty('end')) {
      // Add an offset of +/- 30 days to the date range, to avoid the delayed
      // loading fater display.
      newStart = nextpath_helpers.formatDate(nextpath_helpers.increaseDate(range.start, -30));
      newEnd = nextpath_helpers.formatDate(nextpath_helpers.increaseDate(range.end, 30));
    }
    else if (Array.isArray(range)) {
      // Add an offset of +/- 30 days to the date range, to avoid the delayed
      // loading fater display.
      newStart = nextpath_helpers.formatDate(nextpath_helpers.increaseDate(range[0], -30));
      newEnd = nextpath_helpers.formatDate(nextpath_helpers.increaseDate(range[range.length - 1], 30));
    }

    // If start time of the range is smaller then the start time from the state,
    // update the state to the new start time.
    // If end time of the range is bigger then the end time from the state,
    // update the state to the new end time.
    // After state is updated, check for new bookings.
    if (newStart != null && newEnd != null) {
      var rangeExtended = false;
      var dataStartTime = settings.dataStartTime;
      var dataEndTime = settings.dataEndTime;
      if (newStart < settings.dataStartTime) {
        dataStartTime = newStart;
        rangeExtended = true;
      }
      if (newEnd > settings.dataEndTime) {
        dataEndTime = newEnd;
        rangeExtended = true;
      }
      dispatch(updateDateRange(dataStartTime, dataEndTime));
      dispatch(getBookings(settings.dataUnitId, settings.dataEventType, dataStartTime, dataEndTime, rangeExtended ? null : oldestBooking));
    }
  };
}

export const updateDateRange = (dataStartTime, dataEndTime) => ({
  type: UPDATE_DATE_RANGE,
  payload: {
    dataStartTime,
    dataEndTime,
  },
});
