<?php

namespace Drupal\nextpath_bat_booking\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;

/**
 * Plugin implementation of the 'advanced booking form' formatter.
 *
 * @FieldFormatter(
 *   id = "nextpath_advanced_booking_form",
 *   label = @Translation("Advanced Booking Form"),
 *   description = @Translation("Advanced Booking Form."),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class AdvancedBookingFormatter extends EntityReferenceFormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'event_type' => NULL,
      'render_label' => FALSE,
      'startTime' => '',
      'endTime' => '',
      'refreshInterval' => '',
      'calendarDefaultView' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements['event_type'] = [
      '#type' => 'select',
      '#options' => SimpleBookingFormatter::getEventTypes(),
      '#title' => t('Event type'),
      '#default_value' => $this->getSetting('event_type'),
      '#required' => TRUE,
    ];
    $elements['render_label'] = [
      '#type' => 'checkbox',
      '#title' => t('Render label of unit'),
      '#default_value' => $this->getSetting('render_label'),
    ];
    $elements['startTime'] = [
      '#type' => 'textfield',
      '#title' => t('Start time of the period to load'),
      '#description' => t('The events from the selected period will be loaded on calendar load. Should be defined as relative date to current date e.g.: -1 months'),
      '#default_value' => $this->getSetting('startTime'),
      '#required' => TRUE,
    ];
    $elements['endTime'] = [
      '#type' => 'textfield',
      '#title' => t('End time of the period to load'),
      '#description' => t('The events from the selected period will be loaded on calendar load. Should be defined as relative date to current date e.g.: +1 months'),
      '#default_value' => $this->getSetting('endTime'),
      '#required' => TRUE,
    ];
    $elements['refreshInterval'] = [
      '#type' => 'textfield',
      '#title' => t('Intervals to refresh the event list'),
      '#description' => t('Defined in millisecnds e.g.: 3000. Set it to 0 to disable the automatic refresh.'),
      '#default_value' => $this->getSetting('refreshInterval'),
      '#required' => TRUE,
    ];
    $elements['calendarDefaultView'] = [
      '#type' => 'select',
      '#options' => [
        'month' => 'month',
        'week' => 'week',
        'work_week' => 'work_week',
        'day' => 'day',
        'agenda' => 'agenda',
      ],
      '#title' => t('Calendar default view'),
      '#default_value' => $this->getSetting('calendarDefaultView'),
      '#required' => TRUE,
    ];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $event_types = SimpleBookingFormatter::getEventTypes();
    $event_type = $this->getSetting('event_type');
    $summary[] = t('Event type: @event_type', ['@event_type' => isset($event_types[$event_type]) ? $event_types[$event_type] : $event_type]);
    $startTime = $this->getSetting('startTime');
    $endTime = $this->getSetting('endTime');
    $summary[] = t('Events period, from: @startTime to: @endTime', ['@startTime' => $startTime, '@endTime' => $endTime]);
    $calendarDefaultView = $this->getSetting('calendarDefaultView');
    $summary[] = t('Calendar default view: @calendarDefaultView', ['@calendarDefaultView' => $calendarDefaultView]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $event_type = $this->getSetting('event_type');
    $render_label = $this->getSetting('render_label');

    $startTime = strtotime('-1 months', time());
    if (!empty($this->getSetting('startTime'))) {
      if (strtotime($this->getSetting('startTime'), time())) {
        $startTime = strtotime($this->getSetting('startTime'), time());
      }
    }
    $endTime = strtotime('+1 months', time());
    if (!empty($this->getSetting('endTime'))) {
      if (strtotime($this->getSetting('endTime'), time())) {
        $endTime = strtotime($this->getSetting('endTime'), time());
      }
    }
    $refreshinterval = 0;
    if (!empty($this->getSetting('refreshInterval'))) {
      $refreshinterval = $this->getSetting('refreshInterval');
    }
    $calendarDefaultView = $this->getSetting('calendarDefaultView');

    if (!empty($event_type)) {
      foreach ($this->getEntitiesToView($items, $langcode) as $delta => $entity) {
        if ($entity->id() && $entity->getEntityType()->id() && $entity->getEntityType()->id() == 'bat_unit') {
          if ($render_label) {
            $elements[$delta]['label'] = [
              '#type' => 'container',
              '#markup' => $entity->label(),
              '#attributes' => [
                'class' => 'nextpath_advanced_booking_label',
              ],
            ];
          }
          $elements[$delta]['content'] = [
            '#type' => 'container',
            '#attributes' => [
              'class' => 'nextpath_advanced_booking_form',
              'data-unit-id' => $entity->uuid(),
              'data-event-type' => $event_type,
              'data-states' => json_encode(SimpleBookingFormatter::getEventStates()),
              'data-language' => \Drupal::languageManager()->getCurrentLanguage()->getId(),
              'data-start-time' => date('Y-m-d\TH:i:s', $startTime),
              'data-end-time' => date('Y-m-d\TH:i:s', $endTime),
              'data-refresh-interval' => $refreshinterval,
              'data-calendar-default-view' => $calendarDefaultView,
            ],
          ];
        }
      }
      $elements['#attached']['library'][] = 'nextpath_bat_booking/advanced-booking';
      $elements['#attached']['drupalSettings']['path']['getHost'] = \Drupal::request()->getSchemeAndHttpHost();
      $elements['#attached']['drupalSettings']['user']['uuid'] = User::load(\Drupal::currentUser()->id())->uuid();
    }

    return $elements;
  }

}
