<?php

namespace Drupal\nextpath_bat_booking\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;

/**
 * Plugin implementation of the 'simple booking form' formatter.
 *
 * @FieldFormatter(
 *   id = "nextpath_simple_booking_form",
 *   label = @Translation("Simple Booking Form"),
 *   description = @Translation("Simple Booking Form."),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class SimpleBookingFormatter extends EntityReferenceFormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'event_type' => NULL,
      'event_state' => NULL,
      'render_label' => FALSE,
      'startDateFormat' => 'dd.MM.yyyy HH:mm',
      'startShowTimeSelect' => TRUE,
      'startTimeIntervals' => 15,
      'startTimeFormat' => 'HH:mm',
      'startDateDisable' => FALSE,
      'endDateFormat' => 'dd.MM.yyyy HH:mm',
      'endShowTimeSelect' => TRUE,
      'endShowTimeSelectOnly' => FALSE,
      'endTimeIntervals' => 15,
      'endTimeFormat' => 'HH:mm',
      'endDateDisable' => FALSE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements['event_type'] = [
      '#type' => 'select',
      '#options' => $this->getEventTypes(),
      '#title' => t('Event type'),
      '#default_value' => $this->getSetting('event_type'),
      '#required' => TRUE,
    ];
    $elements['event_state'] = [
      '#type' => 'select',
      '#options' => $this->getEventStates(),
      '#title' => t('Event state'),
      '#default_value' => $this->getSetting('event_state'),
      '#required' => TRUE,
    ];
    $elements['render_label'] = [
      '#type' => 'checkbox',
      '#title' => t('Render label of unit'),
      '#default_value' => $this->getSetting('render_label'),
    ];
    // Settings for the Reactjs Datepicker.
    $elements['startDateFormat'] = [
      '#type' => 'textfield',
      '#title' => t('Date format for the start date input field'),
      '#description' => t('Date format for the start date input field e.g., dd.MM.yyyy HH:mm'),
      '#default_value' => $this->getSetting('startDateFormat'),
      '#required' => TRUE,
    ];
    $elements['startShowTimeSelect'] = [
      '#type' => 'checkbox',
      '#title' => t('Show time selector on start date input'),
      '#default_value' => $this->getSetting('startShowTimeSelect'),
    ];
    $elements['startTimeIntervals'] = [
      '#type' => 'textfield',
      '#title' => t('Intervals to show on start time selector'),
      '#description' => t('Intervals to show on time selector e.g., 15 (meaning the options: 0, 15, 30, 45)'),
      '#default_value' => $this->getSetting('startTimeIntervals'),
      '#required' => TRUE,
    ];
    $elements['startTimeFormat'] = [
      '#type' => 'textfield',
      '#title' => t('Time format for the start time selector'),
      '#description' => t('Time format for the start time selector e.g., HH:mm'),
      '#default_value' => $this->getSetting('startTimeFormat'),
      '#required' => TRUE,
    ];
    $elements['startDateDisable'] = [
      '#type' => 'checkbox',
      '#title' => t('Disable start date input'),
      '#default_value' => $this->getSetting('startDateDisable'),
    ];
    $elements['endDateFormat'] = [
      '#type' => 'textfield',
      '#title' => t('Date format for the end date input field'),
      '#description' => t('Date format for the end date input field e.g., dd.MM.yyyy HH:mm'),
      '#default_value' => $this->getSetting('endDateFormat'),
      '#required' => TRUE,
    ];
    $elements['endShowTimeSelect'] = [
      '#type' => 'checkbox',
      '#title' => t('Show time selector on end date input'),
      '#default_value' => $this->getSetting('endShowTimeSelect'),
    ];
    $elements['endShowTimeSelectOnly'] = [
      '#type' => 'checkbox',
      '#title' => t('Show only the time selector on end date input'),
      '#default_value' => $this->getSetting('endShowTimeSelectOnly'),
    ];
    $elements['endTimeIntervals'] = [
      '#type' => 'textfield',
      '#title' => t('Intervals to show on end time selector'),
      '#description' => t('Intervals to show on end selector e.g., 15 (meaning the options: 0, 15, 30, 45)'),
      '#default_value' => $this->getSetting('endTimeIntervals'),
      '#required' => TRUE,
    ];
    $elements['endTimeFormat'] = [
      '#type' => 'textfield',
      '#title' => t('Time format for the end time selector'),
      '#description' => t('Time format for the end time selector e.g., HH:mm'),
      '#default_value' => $this->getSetting('endTimeFormat'),
      '#required' => TRUE,
    ];
    $elements['endDateDisable'] = [
      '#type' => 'checkbox',
      '#title' => t('Disable end date input'),
      '#default_value' => $this->getSetting('endDateDisable'),
    ];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $event_types = $this->getEventTypes();
    $event_type = $this->getSetting('event_type');
    $summary[] = t('Event type: @event_type', ['@event_type' => isset($event_types[$event_type]) ? $event_types[$event_type] : $event_type]);
    $event_states = $this->getEventStates();
    $event_state = $this->getSetting('event_state');
    $summary[] = t('Event state: @event_state', ['@event_state' => isset($event_states[$event_state]) ? $event_states[$event_state] : $event_state]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $event_type = $this->getSetting('event_type');
    $event_state = $this->getSetting('event_state');
    $render_label = $this->getSetting('render_label');
    if (!empty($event_type) && !empty($event_state)) {
      foreach ($this->getEntitiesToView($items, $langcode) as $delta => $entity) {
        if ($entity->id() && $entity->getEntityType()->id() && $entity->getEntityType()->id() == 'bat_unit') {
          if ($render_label) {
            $elements[$delta]['label'] = [
              '#type' => 'container',
              '#markup' => $entity->label(),
              '#attributes' => [
                'class' => 'nextpath_simple_booking_label',
              ],
            ];
          }
          $elements[$delta]['content'] = [
            '#type' => 'container',
            '#attributes' => [
              'class' => 'nextpath_simple_booking_form',
              'data-unit-id' => $entity->uuid(),
              'data-event-type' => $event_type,
              'data-event-state' => $event_state,
            ],
            '#entity-id' => $entity->id(),
          ];
        }
      }
      $elements['#attached']['library'][] = 'nextpath_bat_booking/simple-booking';
      $elements['#attached']['drupalSettings']['path']['getHost'] = \Drupal::request()->getSchemeAndHttpHost();
      $elements['#attached']['drupalSettings']['nextpath_simple_booking_settings'] = [
        'startDateFormat' => $this->getSetting('startDateFormat'),
        'startShowTimeSelect' => $this->getSetting('startShowTimeSelect'),
        'startTimeIntervals' => $this->getSetting('startTimeIntervals'),
        'startTimeFormat' => $this->getSetting('startTimeFormat'),
        'startDateDisable' => $this->getSetting('startDateDisable'),
        'endDateFormat' => $this->getSetting('endDateFormat'),
        'endShowTimeSelect' => $this->getSetting('endShowTimeSelect'),
        'endShowTimeSelectOnly' => $this->getSetting('endShowTimeSelectOnly'),
        'endTimeIntervals' => $this->getSetting('endTimeIntervals'),
        'endTimeFormat' => $this->getSetting('endTimeFormat'),
        'endDateDisable' => $this->getSetting('endDateDisable'),
      ];
      $elements['#attached']['drupalSettings']['user']['uuid'] = User::load(\Drupal::currentUser()->id())->uuid();
    }
    return $elements;
  }

  /**
   * Get the available Event types.
   */
  public static function getEventTypes() {
    $types = [];
    $eids = \Drupal::entityQuery('bat_event_type')->execute();
    $entity_storage = \Drupal::entityTypeManager()->getStorage('bat_event_type');
    $entities = $entity_storage->loadMultiple($eids);
    foreach ($entities as $entity) {
      $types[$entity->id()] = $entity->label();
    }
    return $types;
  }

  /**
   * Get the available Event states.
   */
  public static function getEventStates() {
    $types = [];
    $types[''] = t('Please select');
    $eids = \Drupal::entityQuery('state')->execute();
    $entity_storage = \Drupal::entityTypeManager()->getStorage('state');
    $entities = $entity_storage->loadMultiple($eids);
    foreach ($entities as $entity) {
      $types[$entity->uuid()] = $entity->label();
    }
    return $types;
  }

}
